from suds.client import Client

from model import *
from model import _Security


class Acquirer(object):

    def __init__(self, affiliation, password, environment):
        self.security = _Security(affiliation, password, environment)

    def authorize_credit(self, request):
        try:
            # create wcf objects
            client = Client(self.__wsdl_path)
            wcf_request = client.factory.create('ns0:GetAuthorizedCredit')
            wcf_request = GetAuthorizedCreditRequest.map(wcf_request, request, self.security)

            # make the request
            result = client.service.GetAuthorizedCredit(wcf_request)

            # mapping the result
            response = GetAuthorizedCreditResponse.map(result)
        except Exception as e:
            response = GetAuthorizedCreditResponse()
            response.return_code = TransactionStatus.transaction_not_processed
            response.messages.append(e.message)

        return response

    def capture(self, capture_request):
        try:
            # create wcf objects
            client = Client(self.__wsdl_path)
            wcf_request = client.factory.create('ns0:ConfirmTxnTID')
            wcf_request = CaptureRequest.map(wcf_request, capture_request, self.security)

            # make the request
            result = client.service.ConfirmTxnTID(wcf_request)

            # mapping the result
            response = CaptureResponse.map(result)
        except Exception as e:
            response = CaptureResponse()
            response.return_code = TransactionStatus.transaction_not_processed
            response.messages.append(e.message)

        return response

    def find(self, find_request):
        try:
            # create wcf objects
            client = Client(self.__wsdl_path)
            wcf_request = client.factory.create('ns0:QueryRequest')
            wcf_request = FindRequest.map(wcf_request, find_request, self.security)

            # make the request
            result = client.service.Query(wcf_request)

            # mapping the result
            response = FindResponse.map(result)
        except Exception as e:
            response = FindResponse()
            response.return_code = TransactionStatus.transaction_not_processed
            response.messages.append(e.message)

        return response

    def cancel(self, cancel_request):
        try:
            # create wcf objects
            client = Client(self.__wsdl_path)
            wcf_request = client.factory.create('ns0:VoidTransactionTID')
            wcf_request = CancelRequest.map(wcf_request, cancel_request, self.security)

            # make the request
            result = client.service.VoidTransactionTID(wcf_request)

            # mapping the result
            response = CancelRequest.map(result)
        except Exception as e:
            response = CancelResponse()
            response.return_code = TransactionStatus.transaction_not_processed
            response.messages.append(e.message)

        return response

    @property
    def __wsdl_path(self):
        url = ""
        if self.security.environment == 0:
            url = "https://scommerce.userede.com.br/Redecard.Komerci.External.WcfKomerci/KomerciWcf.svc?WSDL"
        elif self.security.environment == 1:
            url = "https://ecommerce.userede.com.br/Redecard.Adquirencia.Wcf/KomerciWcf.svc?WSDL"
        else:
            url = "http://localhost:50000/KomerciWcf.svc?WSDL"

        return url

